@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  This file is part of the GtkHTML library.
 
-    Copyright (C) 2000 Helix Code, Inc.
+    Copyright (C) 2000 Helix Code, Inc. 2001, 2002 Ximian Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -19,10 +19,15 @@
     Boston, MA 02111-1307, USA.
 
     Author: Ettore Perazzoli <ettore@helixcode.com>
+            Radek Doulik <rodo@ximian.com>
 
 */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include <string.h>
 #include <gnome.h>
 #include <bonobo.h>
 #include <stdio.h>
@@ -32,8 +37,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <gtk/gtkscrolledwindow.h>
+
 #include <glade/glade.h>
-#include <gal/widgets/e-scroll-frame.h>
 
 #ifdef GNOME_GTKHTML_EDITOR_SHLIB
 #include <bonobo/bonobo-shlib-factory.h>
@@ -46,8 +52,11 @@
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
+#include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-selection-updater.h"
+#include "htmlimage.h"
+#include "htmlinterval.h"
 #include "htmlselection.h"
 #include "htmlfontmanager.h"
 #include "htmlsettings.h"
@@ -56,8 +65,8 @@
 
 #include "engine.h"
 #include "menubar.h"
-#include "persist-file-impl.h"
-#include "persist-stream-impl.h"
+#include "persist-file.h"
+#include "persist-stream.h"
 #include "popup.h"
 #include "toolbar.h"
 #include "properties.h"
@@ -68,18 +77,8 @@
 #include "html-stream-mem.h"
 
 #include "gtkhtmldebug.h"
+#include "editor-control-factory.h"
 
-#ifdef USING_OAF
-#define CONTROL_FACTORY_ID "OAFIID:GNOME_GtkHTML_Editor_Factory:" EDITOR_API_VERSION
-#else
-#define CONTROL_FACTORY_ID "control-factory:html-editor"
-#endif
-
-#ifndef GNOME_GTKHTML_EDITOR_SHLIB
-static BonoboGenericFactory *factory;
-static gint active_controls = 0;
-
-#endif
 static void send_event_stream (GNOME_GtkHTML_Editor_Engine engine, 
 			       GNOME_GtkHTML_Editor_Listener listener,
 			       const char *name,
@@ -98,13 +97,6 @@ typedef struct _SetFrameData SetFrameData;
 
 static GtkHTMLEditorAPI *editor_api;
 
-static gint
-gtk_toolbar_focus (GtkContainer     *container,
-			  GtkDirectionType  direction)
-{
-	return FALSE;
-}
-
 static void
 set_frame_cb (BonoboControl *control,
 	      gpointer data)
@@ -113,33 +105,32 @@ set_frame_cb (BonoboControl *control,
 	BonoboUIComponent *ui_component;
 	GtkHTMLControlData *control_data;
 	GtkWidget *toolbar;
-	GtkWidget *scroll_frame;
+	GtkWidget *scrolled_window;
+	Bonobo_ControlFrame frame;
 
 	control_data = (GtkHTMLControlData *) data;
 
-	if (bonobo_control_get_control_frame (control) == CORBA_OBJECT_NIL)
+	frame = bonobo_control_get_control_frame (control, NULL);
+	if (frame == CORBA_OBJECT_NIL)
 		return;
+	CORBA_Object_release (frame, NULL);
 
-	remote_ui_container = bonobo_control_get_remote_ui_container (control);
+	remote_ui_container = bonobo_control_get_remote_ui_container (control, NULL);
 	control_data->uic = ui_component = bonobo_control_get_ui_component (control);
-	bonobo_ui_component_set_container (ui_component, remote_ui_container);
+	bonobo_ui_component_set_container (ui_component, remote_ui_container, NULL);
 
 	/* Setup the tool bar.  */
 
 	toolbar = toolbar_style (control_data);
-
-	/* hack to fix gtk toolbar focus bug, should be removed once gtk+ is fixed */
-	if (GTK_CONTAINER_CLASS (gtk_type_class (gtk_toolbar_get_type ()))->focus == NULL)
-		GTK_CONTAINER_CLASS (gtk_type_class (gtk_toolbar_get_type ()))->focus = gtk_toolbar_focus;
 	gtk_box_pack_start (GTK_BOX (control_data->vbox), toolbar, FALSE, FALSE, 0);
 
-	scroll_frame = e_scroll_frame_new (NULL, NULL);
-	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (scroll_frame), GTK_SHADOW_IN);
-	e_scroll_frame_set_policy (E_SCROLL_FRAME (scroll_frame), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_container_add (GTK_CONTAINER (scroll_frame), GTK_WIDGET (control_data->html));
-	gtk_widget_show_all (scroll_frame);
+	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+	gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (control_data->html));
+	gtk_widget_show_all (scrolled_window);
 
-	gtk_box_pack_start (GTK_BOX (control_data->vbox), scroll_frame, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (control_data->vbox), scrolled_window, TRUE, TRUE, 0);
 
 	/* Setup the menu bar.  */
 
@@ -152,6 +143,8 @@ set_frame_cb (BonoboControl *control,
 		control_data->has_spell_control = TRUE;
 
 	gtk_html_set_editor_api (GTK_HTML (control_data->html), editor_api, control_data);
+
+	bonobo_object_release_unref (remote_ui_container, NULL);
 }
 
 static gint
@@ -227,7 +220,7 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 			gtk_html_edit_properties_dialog_set_page (cd->properties_dialog, start);
 		}
 	}
-	gtk_signal_disconnect (GTK_OBJECT (widget), cd->releaseId);
+	g_signal_handler_disconnect (widget, cd->releaseId);
 
 	return FALSE;
 }
@@ -242,10 +235,8 @@ load_from_file (GtkHTML *html,
 	int fd;
         const char *path;
 
-        if (strncmp (url, "file:", 5) != 0) {
-		return FALSE;
-	} 
-	path = url + 5; 
+        if (strncmp (url, "file:", 5) == 0)
+		path = url + 5; 
 
 	if ((fd = open (path, O_RDONLY)) == -1) {
 		g_warning ("%s", g_strerror (errno));
@@ -310,15 +301,15 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 					     &offset, FALSE);
 	switch (event->button) {
 	case 1:
-		if (event->type == GDK_2BUTTON_PRESS && cd->obj && event->state & GDK_CONTROL_MASK)
-			cd->releaseId = gtk_signal_connect (GTK_OBJECT (html), "button_release_event",
-							    GTK_SIGNAL_FUNC (release), cd);
-		else
+		if (event->type == GDK_2BUTTON_PRESS && cd->obj && event->state & GDK_CONTROL_MASK) {
+			cd->releaseId = g_signal_connect (html, "button_release_event",
+							  G_CALLBACK (release), cd);
 			return TRUE;
+		}
 		break;
 	case 2:
 		/* pass this for pasting */
-		return TRUE;
+		return FALSE;
 	case 3:
 		if (!html_engine_is_selection_active (engine) || !html_engine_point_in_selection (engine, cd->obj, offset)) {
 			html_engine_disable_selection (engine);
@@ -328,8 +319,10 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 			gtk_html_update_styles (cd->html);
 		}
 
-		if (popup_show (cd, event))
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (html), "button_press_event");
+		if (popup_show (cd, event)) {
+			g_signal_stop_emission_by_name (html, "button_press_event");
+			return TRUE;
+		}
 		break;
 	default:
 		;
@@ -338,20 +331,28 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 	return FALSE;
 }
 
-static void
-destroy_control_data_cb (GtkObject *control, GtkHTMLControlData *cd)
+static gint
+html_button_pressed_after (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData *cd)
 {
-	gtk_html_control_data_destroy (cd);
+	HTMLEngine *e = cd->html->engine;
+	HTMLObject *obj = e->cursor->object;
 
-#ifndef GNOME_GTKHTML_EDITOR_SHLIB
-	active_controls --;
+	if (event->button == 1 && event->type == GDK_BUTTON_PRESS && obj && obj->parent && !html_engine_is_selection_active (e)) {
+		if (html_object_is_text (obj) && html_object_get_data (obj->parent, "template_text")) {
+			html_object_set_data_full (obj->parent, "template_text", NULL, NULL);
+			html_cursor_jump_to_position (e->cursor, e, e->cursor->position - e->cursor->offset);
+			html_engine_set_mark (e);
+			html_cursor_jump_to_position (e->cursor, e, e->cursor->position + html_object_get_length (obj));
+			html_engine_select_interval (e, html_interval_new_from_cursor (e->mark, e->cursor));
+			/* printf ("delete template text\n"); */
+			html_engine_delete (cd->html->engine);
+		} else if (HTML_IS_IMAGE (obj) && html_object_get_data (obj->parent, "template_image")) {
+			property_dialog_show (cd);
+			html_object_set_data_full (obj->parent, "template_image", NULL, NULL);
+		}
+	}
 
-	if (active_controls)
-		return;
-
-	bonobo_object_unref (BONOBO_OBJECT (factory));
-	gtk_main_quit ();
-#endif
+	return FALSE;
 }
 
 static void
@@ -363,12 +364,11 @@ editor_init_painters (GtkHTMLControlData *cd)
 	g_return_if_fail (cd != NULL);
 
 	html = cd->html;
-	prop = GTK_HTML_CLASS (GTK_OBJECT (html)->klass)->properties;
+	prop = GTK_HTML_CLASS (GTK_OBJECT_GET_CLASS (html))->properties;
 	
 	if (!cd->plain_painter) {
-		cd->plain_painter = HTML_GDK_PAINTER (html_plain_painter_new (TRUE));
-		html_font_manager_set_default (html_engine_font_manager_with_painter (html->engine,
-										      HTML_PAINTER (cd->plain_painter)),
+		cd->plain_painter = HTML_GDK_PAINTER (html_plain_painter_new (GTK_WIDGET (html), TRUE));
+		html_font_manager_set_default (&HTML_PAINTER (cd->plain_painter)->font_manager,
 					       prop->font_var,      prop->font_fix,
 					       prop->font_var_size, prop->font_var_points,
 					       prop->font_fix_size, prop->font_fix_points);
@@ -379,8 +379,8 @@ editor_init_painters (GtkHTMLControlData *cd)
 		cd->gdk_painter = HTML_GDK_PAINTER (html->engine->painter);
 
 		/* the plain painter starts with a ref */
-		gtk_object_ref (GTK_OBJECT (cd->gdk_painter));
-	}
+		g_object_ref (G_OBJECT (cd->gdk_painter));
+	}	
 }
 
 static void
@@ -404,6 +404,8 @@ editor_set_format (GtkHTMLControlData *cd, gboolean format_html)
 		old_p = cd->gdk_painter;
 	}		
 
+	printf ("set format %d\n", format_html);
+
 	toolbar_update_format (cd);
 	menubar_update_format (cd);
 
@@ -468,6 +470,12 @@ editor_set_prop (BonoboPropertyBag *bag,
 	}
 }
 
+static void
+control_destroy (BonoboObject *o, GtkHTMLControlData *cd)
+{
+	gtk_html_control_data_destroy (cd);
+}
+
 static void
 editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 {
@@ -476,15 +484,13 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	BonoboPropertyBag  *pb;
 	BonoboArg          *def;
 
-#ifndef GNOME_GTKHTML_EDITOR_SHLIB
-	active_controls++;
-#endif
 	/* GtkHTML widget */
 	html_widget = gtk_html_new ();
 	gtk_html_load_empty (GTK_HTML (html_widget));
 	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
 
 	cd = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
+	g_signal_connect (control, "destroy", G_CALLBACK (control_destroy), cd);
 
 	/* HTMLEditor::Engine */
 	cd->editor_bonobo_engine = editor_engine_new (cd);
@@ -492,16 +498,15 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 				     BONOBO_OBJECT (cd->editor_bonobo_engine));
 
 	/* Bonobo::PersistStream */
-	cd->persist_stream = persist_stream_impl_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (cd->persist_stream));
+	cd->persist_stream = gtk_html_persist_stream_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), cd->persist_stream);
 
 	/* Bonobo::PersistFile */
-	cd->persist_file = persist_file_impl_new (GTK_HTML (html_widget));
-	bonobo_object_add_interface (BONOBO_OBJECT (control), BONOBO_OBJECT (cd->persist_file));
+	cd->persist_file = gtk_html_persist_file_new (GTK_HTML (html_widget));
+	bonobo_object_add_interface (BONOBO_OBJECT (control), cd->persist_file);
 
 	/* PropertyBag */
 	pb = bonobo_property_bag_new (editor_get_prop, editor_set_prop, cd);
-	bonobo_control_set_properties (control, pb);
 
 	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
 	BONOBO_ARG_SET_BOOLEAN (def, TRUE);
@@ -531,23 +536,17 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 				 0);
 	*/
 
+	bonobo_control_set_properties (control, BONOBO_OBJREF (pb), NULL);
 	bonobo_object_unref (BONOBO_OBJECT (pb));
 
 	/* Part of the initialization must be done after the control is
 	   embedded in its control frame.  We use the "set_frame" signal to
 	   handle that.  */
 
-	gtk_signal_connect (GTK_OBJECT (control), "set_frame",
-			    GTK_SIGNAL_FUNC (set_frame_cb), cd);
-
-	gtk_signal_connect (GTK_OBJECT (control), "destroy",
-			    GTK_SIGNAL_FUNC (destroy_control_data_cb), cd);
-
-	gtk_signal_connect (GTK_OBJECT (html_widget), "url_requested",
-			    GTK_SIGNAL_FUNC (url_requested_cb), cd);
-
-	gtk_signal_connect (GTK_OBJECT (html_widget), "button_press_event",
-			    GTK_SIGNAL_FUNC (html_button_pressed), cd);
+	g_signal_connect (control, "set_frame", G_CALLBACK (set_frame_cb), cd);
+	g_signal_connect (html_widget, "url_requested", G_CALLBACK (url_requested_cb), cd);
+	g_signal_connect (html_widget, "button_press_event", G_CALLBACK (html_button_pressed), cd);
+	g_signal_connect_after (html_widget, "button_press_event", G_CALLBACK (html_button_pressed_after), cd);
 
 	cd->control = control;
 }
@@ -575,40 +574,15 @@ editor_api_command (GtkHTML *html, GtkHTMLCommandType com_type, gpointer data)
 	return rv;
 }
 
-static GtkType
-wait_for_bonobo_patch__bonobo_arg_type_to_gtk (BonoboArgType id)
+static GValue *
+send_event_str (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listener listener, gchar *name, GValue *arg)
 {
 	CORBA_Environment ev;
-	GtkType gtk_type = GTK_TYPE_NONE;
-
-	CORBA_exception_init (&ev);
-
-	if (bonobo_arg_type_is_equal (TC_char, id, &ev))         gtk_type = GTK_TYPE_CHAR;
-	else if (bonobo_arg_type_is_equal (TC_boolean, id, &ev)) gtk_type = GTK_TYPE_BOOL;
-	else if (bonobo_arg_type_is_equal (TC_short,   id, &ev)) gtk_type = GTK_TYPE_INT;
-	else if (bonobo_arg_type_is_equal (TC_ushort,  id, &ev)) gtk_type = GTK_TYPE_UINT;
-	else if (bonobo_arg_type_is_equal (TC_long,    id, &ev)) gtk_type = GTK_TYPE_LONG;
-	else if (bonobo_arg_type_is_equal (TC_ulong,   id, &ev)) gtk_type = GTK_TYPE_ULONG;
-	else if (bonobo_arg_type_is_equal (TC_float,   id, &ev)) gtk_type = GTK_TYPE_FLOAT;
-	else if (bonobo_arg_type_is_equal (TC_double,  id, &ev)) gtk_type = GTK_TYPE_DOUBLE;
-	else if (bonobo_arg_type_is_equal (TC_string,  id, &ev)) gtk_type = GTK_TYPE_STRING;
-	else
-		g_warning ("Unmapped bonobo arg type");
-
-	CORBA_exception_free (&ev);
-
-	return gtk_type;
-}
-
-static GtkArg *
-send_event_str (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listener listener, gchar *name, GtkArg *arg)
-{
-	CORBA_Environment ev;
-	GtkArg *gtk_retval = NULL;
-	BonoboArg *bonobo_arg = bonobo_arg_new (bonobo_arg_type_from_gtk (arg->type));
+	GValue *gvalue_retval = NULL;
+	BonoboArg *bonobo_arg = bonobo_arg_new (bonobo_arg_type_from_gtype (G_VALUE_TYPE (arg)));
 	BonoboArg *bonobo_retval;
 
-	bonobo_arg_from_gtk (bonobo_arg, arg);
+	bonobo_arg_from_gvalue (bonobo_arg, arg);
 
 	/* printf ("sending to listener\n"); */
 	CORBA_exception_init (&ev);
@@ -618,14 +592,15 @@ send_event_str (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listene
 	if (ev._major == CORBA_NO_EXCEPTION) {
 		if (!bonobo_arg_type_is_equal (bonobo_retval->_type, TC_null, &ev)
 		    && !bonobo_arg_type_is_equal (bonobo_retval->_type, TC_void, &ev)) {
-			gtk_retval = gtk_arg_new (wait_for_bonobo_patch__bonobo_arg_type_to_gtk (bonobo_retval->_type));
-			bonobo_arg_to_gtk (gtk_retval, bonobo_retval);
+			gvalue_retval = g_new0 (GValue, 1);
+			gvalue_retval = g_value_init (gvalue_retval, bonobo_arg_type_to_gtype (bonobo_retval->_type));
+			bonobo_arg_to_gvalue (gvalue_retval, bonobo_retval);
 		}
 		CORBA_free (bonobo_retval);
 	}
 	CORBA_exception_free (&ev);
 
-	return gtk_retval;
+	return gvalue_retval;
 }
 
 static void
@@ -652,7 +627,7 @@ send_event_stream (GNOME_GtkHTML_Editor_Engine engine,
 	CORBA_any *any;
 	GNOME_GtkHTML_Editor_URLRequestEvent e;
 	CORBA_Environment ev;
-	BonoboStream *bstream;
+	BonoboObject *bstream;
 
 	any = CORBA_any__alloc ();
 	any->_type = TC_GNOME_GtkHTML_Editor_URLRequestEvent;
@@ -670,11 +645,11 @@ send_event_stream (GNOME_GtkHTML_Editor_Engine engine,
 	CORBA_free (any);
 }
 		  
-static GtkArg *
-editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **args, gpointer data)
+static GValue *
+editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GValue *args, gpointer data)
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
-	GtkArg *gtk_retval = NULL;
+	GValue *retval = NULL;
 
 	/* printf ("editor_api_event\n"); */
 
@@ -691,13 +666,13 @@ editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **arg
 
 			switch (event_type) {
 			case GTK_HTML_EDITOR_EVENT_COMMAND_BEFORE:
-				gtk_retval = send_event_str (engine, listener, "command_before", args [0]);
+				retval = send_event_str (engine, listener, "command_before", args);
 				break;
 			case GTK_HTML_EDITOR_EVENT_COMMAND_AFTER:
-				gtk_retval = send_event_str (engine, listener, "command_after", args [0]);
+				retval = send_event_str (engine, listener, "command_after", args);
 				break;
 			case GTK_HTML_EDITOR_EVENT_IMAGE_URL:
-				gtk_retval = send_event_str (engine, listener, "image_url", args [0]);
+				retval = send_event_str (engine, listener, "image_url", args);
 				break;
 			case GTK_HTML_EDITOR_EVENT_DELETE:
 				send_event_void (engine, listener, "delete");
@@ -708,7 +683,7 @@ editor_api_event (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **arg
 			CORBA_exception_free (&ev);
 		}
 	}
-	return gtk_retval;
+	return retval;
 }
 
 static GtkWidget *
@@ -748,17 +723,18 @@ editor_control_init (void)
 		initialized = TRUE;
 
 		new_editor_api ();
-		gdk_rgb_init ();
-		glade_gnome_init ();
+		glade_init ();
 	}
 }
 
 static BonoboObject *
-editor_control_factory (BonoboGenericFactory *factory, gpointer closure)
+editor_control_factory (BonoboGenericFactory *factory, const gchar *component_id, gpointer closure)
 {
 	BonoboControl *control;
 	GtkWidget *vbox;
 
+	printf ("factory: %s\n", component_id);
+
 	editor_control_init ();
 
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -782,55 +758,10 @@ BONOBO_OAF_SHLIB_FACTORY (CONTROL_FACTORY_ID, "GNOME HTML Editor factory", edito
 
 #else
 
-#ifdef USING_OAF
-
-#include <liboaf/liboaf.h>
-
-static CORBA_ORB
-init_corba (int *argc, char **argv)
-{
-	gnome_init_with_popt_table ("gnome-gtkhtml-editor", VERSION,
-				    *argc, argv, oaf_popt_options, 0, NULL);
-
-	return oaf_init (*argc, argv);
-}
-
-#else
-
-#include <libgnorba/gnorba.h>
-
-static CORBA_ORB
-init_corba (int *argc, char **argv)
-{
-	CORBA_Environment ev;
-
-	CORBA_exception_init (&ev);
-
-	gnome_CORBA_init_with_popt_table ("html-editor-factory", EDITOR_API_VERSION,
-					  argc, argv,
-					  NULL, 0, NULL,
-					  GNORBA_INIT_SERVER_FUNC,
-					  &ev);
-	if (ev._major != CORBA_NO_EXCEPTION)
-		g_error (_("Could not initialize GNORBA"));
-
-	CORBA_exception_free (&ev);
-
-	return CORBA_OBJECT_NIL;
-}
-
-#endif
-
-static void
-init_bonobo (int *argc, char **argv)
-{
-	if (bonobo_init (init_corba (argc, argv), CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
-		g_error (_("Could not initialize Bonobo"));
-}
-
 int
 main (int argc, char **argv)
 {
+	BonoboGenericFactory *factory;
 #ifdef GTKHTML_HAVE_GCONF
 	GError  *gconf_error  = NULL;
 #endif
@@ -839,23 +770,30 @@ main (int argc, char **argv)
 	bindtextdomain(GTKHTML_RELEASE_STRING, GNOMELOCALEDIR);
 	textdomain(GTKHTML_RELEASE_STRING);
 
-	init_bonobo (&argc, argv);
-#ifdef GTKHTML_HAVE_GCONF
+	gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
+			   GNOME_PROGRAM_STANDARD_PROPERTIES,
+			   GNOME_PARAM_HUMAN_READABLE_NAME, _("GtkHTML Editor Control"),			   
+			   NULL);
+
+	/* #ifdef GTKHTML_HAVE_GCONF
 	if (!gconf_init (argc, argv, &gconf_error)) {
 		g_assert (gconf_error != NULL);
 		g_error ("GConf init failed:\n  %s", gconf_error->message);
 		return 1;
 	}
-#endif
+	#endif */
 
-	factory = bonobo_generic_factory_new (CONTROL_FACTORY_ID,
-					      editor_control_factory,
-					      NULL);
-	if (factory == NULL)
-		g_error ("I could not register the GNOME_GtkHTML_Editor factory.");
+	factory = bonobo_generic_factory_new (CONTROL_FACTORY_ID, editor_control_factory, NULL);
 
-	bonobo_main ();
+	if (factory) {
+		bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
+	
+		bonobo_activate ();
+		bonobo_main ();
+
+		return bonobo_ui_debug_shutdown ();
+	} else
+		return 1;
 
-	return 0;
 }
 #endif /* GNOME_GTKHTML_EDITOR_SHLIB */
