@@ -23,11 +23,9 @@
 #define _GTKHTML_H_
 
 #include <sys/types.h>
-#include <gtk/gtkbindings.h>
 #include <gtk/gtklayout.h>
-#include <libgnome/gnome-paper.h>
 #include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-master.h>
+#include <libgnomeprint/gnome-print-job.h>
 
 #include "gtkhtml-types.h"
 #include "gtkhtml-enums.h"
@@ -36,15 +34,14 @@
 #include "htmltypes.h"
 
 #define GTK_TYPE_HTML                  (gtk_html_get_type ())
-#define GTK_HTML(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_HTML, GtkHTML))
-#define GTK_HTML_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_HTML, GtkHTMLClass))
-#define GTK_IS_HTML(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_HTML))
-#define GTK_IS_HTML_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_HTML))
+#define GTK_HTML(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_HTML, GtkHTML))
+#define GTK_HTML_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_HTML, GtkHTMLClass))
+#define GTK_IS_HTML(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_HTML))
+#define GTK_IS_HTML_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_HTML))
 
 struct _GtkHTML {
 	GtkLayout layout;
 
-	GtkBindingSet        *editor_bindings;
 	GtkWidget            *iframe_parent;
 	HTMLObject           *frame;
 	GtkHTMLEditorAPI     *editor_api;
@@ -57,13 +54,12 @@ struct _GtkHTML {
 
 	/* The cursors we use within the widget.  */
 	GdkCursor *hand_cursor;
-	GdkCursor *arrow_cursor;
 	GdkCursor *ibeam_cursor;
 
 	gint selection_x1, selection_y1;
 
 	guint in_selection : 1;
-	guint button1_pressed : 1;
+	guint in_selection_drag : 1;
 
 	guint debug : 1;
 	guint allow_selection : 1;
@@ -113,14 +109,14 @@ struct _GtkHTMLEditorAPI
 {
 	/* spell checking methods */
 	gboolean  (* check_word)              (GtkHTML *html, const gchar *word, gpointer data);
-	void      (* suggestion_request)      (GtkHTML *html, const gchar *word, gpointer data);
+	void      (* suggestion_request)      (GtkHTML *html, gpointer data);
 	void      (* add_to_session)          (GtkHTML *html, const gchar *word, gpointer data);
 	void      (* add_to_personal)         (GtkHTML *html, const gchar *word, gpointer data);
 
 	/* unhandled commands */
 	gboolean  (* command)                 (GtkHTML *html, GtkHTMLCommandType com_type, gpointer data);
 
-	GtkArg  * (* event)                   (GtkHTML *html, GtkHTMLEditorEventType event_type, GtkArg **args, gpointer data);
+	GValue  * (* event)                   (GtkHTML *html, GtkHTMLEditorEventType event_type, GValue *args, gpointer data);
 
 	/* input line */
 	GtkWidget * (* create_input_line)     (GtkHTML *html, gpointer data);
@@ -133,8 +129,6 @@ struct _GtkHTMLEditorAPI
 GtkType                    gtk_html_get_type                      (void);
 void                       gtk_html_construct                     (GtkWidget                 *html);
 GtkWidget                 *gtk_html_new                           (void);
-GtkWidget                 *gtk_html_new_from_string               (const gchar               *str,
-								   gint                       len);
 void                       gtk_html_set_editor_api                (GtkHTML                   *html,
 								   GtkHTMLEditorAPI          *api,
 								   gpointer                   data);
@@ -163,8 +157,11 @@ int                        gtk_html_request_paste                 (GtkHTML
 								   gboolean                   as_cite);
 /* Loading.  */
 GtkHTMLStream             *gtk_html_begin                         (GtkHTML                   *html);
-GtkHTMLStream             *gtk_html_begin_content                 (GtkHTML                   *html,
-								   gchar                     *content_type);
+GtkHTMLStream             *gtk_html_begin_full                    (GtkHTML                   *html,
+								   char                      *target_frame,
+								   char                      *content_type,
+								   GtkHTMLBeginFlags         flags);
+
 void                       gtk_html_write                         (GtkHTML                   *html,
 								   GtkHTMLStream             *handle,
 								   const gchar               *buffer,
@@ -172,15 +169,11 @@ void                       gtk_html_write                         (GtkHTML
 void                       gtk_html_end                           (GtkHTML                   *html,
 								   GtkHTMLStream             *handle,
 								   GtkHTMLStreamStatus        status);
-void                       gtk_html_load_empty                    (GtkHTML                   *html);
 void                       gtk_html_load_from_string              (GtkHTML                   *html,
 								   const gchar               *str,
 								   gint                       len);
 
 /* Saving.  */
-gboolean                   gtk_html_save                          (GtkHTML                   *html,
-								   GtkHTMLSaveReceiverFn      receiver,
-								   gpointer                   data);
 gboolean                   gtk_html_export                        (GtkHTML                   *html,
 								   const char                *type,
 								   GtkHTMLSaveReceiverFn      receiver,
@@ -206,7 +199,7 @@ gint                       gtk_html_print_get_pages_num           (GtkHTML
 								   gdouble                    header_height,
 								   gdouble                    footer_height);
 void                       gtk_html_print_set_master              (GtkHTML                   *html,
-								   GnomePrintMaster          *print_master);
+								   GnomePrintJob             *print_master);
 
 /* Title.  */
 const gchar               *gtk_html_get_title                     (GtkHTML                   *html);
@@ -222,7 +215,7 @@ GtkHTMLParagraphStyle      gtk_html_get_paragraph_style           (GtkHTML
 void                       gtk_html_set_paragraph_style           (GtkHTML                   *html,
 								   GtkHTMLParagraphStyle      style);
 void                       gtk_html_set_indent                    (GtkHTML                   *html,
-								   GByteArray                 *levels);
+								   GByteArray                *levels);
 void                       gtk_html_indent_push_level             (GtkHTML                   *html,
 								   HTMLListType               level_type);
 void                       gtk_html_indent_pop_level              (GtkHTML                   *html);
@@ -251,16 +244,12 @@ void                       gtk_html_append_html                   (GtkHTML
 								   const gchar               *html_src);
 
 /* misc utils */
-void                       gtk_html_set_default_background_color  (GtkHTML                   *html,
-								   GdkColor                  *c);
-void                       gtk_html_set_default_content_type      (GtkHTML                   *html,
-								   gchar                     *content_type);
 gpointer                   gtk_html_get_object_by_id              (GtkHTML                   *html,
 								   const gchar               *id);
 gboolean                   gtk_html_command                       (GtkHTML                   *html,
 								   const gchar               *command_name);
 gboolean                   gtk_html_edit_make_cursor_visible      (GtkHTML                   *html);
-gboolean                   gtk_html_build_with_gconf              (void);
+
 void                       gtk_html_set_magnification             (GtkHTML                   *html,
 								   gdouble                    magnification);
 void                       gtk_html_zoom_in                       (GtkHTML                   *html);
@@ -290,4 +279,22 @@ void                       gtk_html_set_blocking                  (GtkHTML
 								   gboolean                   block);
 gboolean                   gtk_html_has_undo                      (GtkHTML                   *html);
 void                       gtk_html_drop_undo                     (GtkHTML                   *html);
+
+/* DEPRECATED */
+#if 1
+gboolean                   gtk_html_build_with_gconf              (void);
+void                       gtk_html_set_default_background_color  (GtkHTML                   *html,
+								   GdkColor                  *c);
+void                       gtk_html_set_default_content_type      (GtkHTML                   *html,
+								   gchar                     *content_type);
+GtkWidget                 *gtk_html_new_from_string               (const gchar               *Astr,
+								   gint                       len);
+void                       gtk_html_load_empty                    (GtkHTML                   *html);
+gboolean                   gtk_html_save                          (GtkHTML                   *html,
+								   GtkHTMLSaveReceiverFn      receiver,
+								   gpointer                   data);
+GtkHTMLStream             *gtk_html_begin_content                 (GtkHTML                   *html,
+								   gchar                     *content_type);
+#endif
+
 #endif /* _GTKHTML_H_ */
