@@ -1,8 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  This file is part of the GtkHTML library.
 
-    Copyright (C) 2000, 2001 Helix Code, Inc.
-    Authors:                 Radek Doulik (rodo@helixcode.com)
+    Copyright (C) 2000, 2001, 2002 Ximian, Inc.
+    Authors:                       Radek Doulik (rodo@ximian.com)
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -30,6 +30,7 @@
 #include "gtkhtml-properties.h"
 
 #include "htmlclue.h"
+#include "htmlcluealigned.h"
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmlcolorset.h"
@@ -47,6 +48,7 @@
 #include "htmlobject.h"
 #include "htmlplainpainter.h"
 #include "htmltable.h"
+#include "htmltablecell.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
 #include "htmltext.h"
@@ -55,8 +57,8 @@
 
 static gint        delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoDirection dir,
 				  gboolean add_prop);
-static void        insert_object (HTMLEngine *e, HTMLObject *obj, guint len, guint position_after, gint level,
-				  HTMLUndoDirection dir, gboolean check);
+static void        insert_object_for_undo (HTMLEngine *e, HTMLObject *obj, guint len, guint position_after, gint level,
+					   HTMLUndoDirection dir, gboolean check);
 static void        append_object (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir);
 static void        insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo);
 static void        insert_setup_undo (HTMLEngine *e, guint len, guint position_before, HTMLUndoDirection dir,
@@ -193,7 +195,8 @@ prepare_delete_bounds (HTMLEngine *e, GList **from_list, GList **to_list,
 	html_point_get_left  (&e->selection->to,   &end);
 
 	common_parent = get_common_parent (begin.object, end.object);
-	ret_level     = get_parent_depth (begin.object, common_parent);
+	ret_level     = html_object_get_parent_level (common_parent);
+	/* printf ("common parent level: %d\n", ret_level); */
 
 	*from_list = point_get_parent_list (&begin, get_parent_depth (begin.object, common_parent), TRUE);
 	*to_list   = point_get_parent_list (&end,   get_parent_depth (end.object, common_parent),   TRUE);
@@ -226,7 +229,7 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 	HTMLObject *lo, *ro, *prev;
 
 #ifdef OP_DEBUG
-	HTMLObject *left_orig = left->data;
+	/* HTMLObject *left_orig = left->data; */
 	printf ("before merge\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
 	if (left && left->data) {
@@ -247,6 +250,37 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 		left  = left->next;
 		right = right->next;
 
+		if (HTML_IS_CLUEALIGNED (lo) && !HTML_IS_CLUEALIGNED (ro) && html_object_is_text (HTML_CLUE (lo)->head)) {
+			HTMLObject *nlo = lo->prev;
+
+			if (e->cursor->object->parent && e->cursor->object->parent == lo) {
+				e->cursor->object = ro;
+				e->cursor->offset = 0;
+			}
+			if (c && c->object->parent && c->object->parent == lo) {
+				c->object = ro;
+				c->offset = 0;
+			}
+
+			html_object_remove_child (lo->parent, lo);
+			html_object_destroy (lo);
+			lo = nlo;
+			if (!nlo)
+				break;
+		} else if (HTML_IS_CLUEALIGNED (ro) && !HTML_IS_CLUEALIGNED (lo) && html_object_is_text (HTML_CLUE (ro)->head)) {
+			HTMLObject *nro = ro->next;
+
+			if (e->cursor->object->parent && e->cursor->object->parent == ro) {
+				e->cursor->object = lo;
+				e->cursor->offset = html_object_get_length (lo);
+			}
+			html_object_remove_child (ro->parent, ro);
+			html_object_destroy (ro);
+			ro = nro;
+			if (!nro)
+				break;
+		}
+
 		if (html_object_is_text (lo) && !*HTML_TEXT (lo)->text && (html_object_prev_not_slave (lo) || merge)) {
 			HTMLObject *nlo = html_object_prev_not_slave (lo);
 
@@ -291,8 +325,8 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 		e->cursor->offset = html_object_get_length (e->cursor->object);
 	}
 #ifdef OP_DEBUG
-	printf ("-- finished\n");
-	gtk_html_debug_dump_tree_simple (left_orig, 0);
+	/* printf ("-- finished\n");
+	   gtk_html_debug_dump_tree_simple (left_orig, 0); */
 	printf ("-- after\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
 	printf ("-- END merge\n");
@@ -380,7 +414,7 @@ delete_undo_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, gu
 
 	undo         = (DeleteUndo *) data;
 	buffer       = html_object_op_copy (undo->buffer, NULL, e, NULL, NULL, &len);
-	insert_object (e, buffer, undo->buffer_len, position_after, undo->level, html_undo_direction_reverse (dir), TRUE);
+	insert_object_for_undo (e, buffer, undo->buffer_len, position_after, undo->level, html_undo_direction_reverse (dir), TRUE);
 }
 
 static void
@@ -390,6 +424,7 @@ delete_setup_undo (HTMLEngine *e, HTMLObject *buffer, guint len, guint position_
 
 	undo = g_new (DeleteUndo, 1);
 
+	/* printf ("cursor level: %d undo level: %d\n", html_object_get_parent_level (e->cursor->object), level); */
 	html_undo_data_init (HTML_UNDO_DATA (undo));
 	undo->data.destroy = delete_undo_destroy;
 	undo->buffer       = buffer;
@@ -473,7 +508,9 @@ check_flows (HTMLEngine *e, HTMLUndoDirection dir)
 	g_return_if_fail (e->mark->object->parent);
 	g_return_if_fail (e->cursor->position <= e->mark->position);
 
-	if (!HTML_IS_CLUEFLOW (e->cursor->object->parent) || !HTML_IS_CLUEFLOW (e->mark->object->parent))
+	if (e->cursor->offset || e->cursor->object->parent == e->mark->object->parent
+	    || !HTML_IS_CLUEFLOW (e->cursor->object->parent) || !HTML_IS_CLUEFLOW (e->mark->object->parent)
+	    || e->cursor->object != HTML_CLUE (e->cursor->object->parent)->head)
 		return;
 
 	level1 = html_object_get_parent_level (e->cursor->object->parent);
@@ -489,13 +526,8 @@ check_flows (HTMLEngine *e, HTMLUndoDirection dir)
 		|| !haligns_equal (HTML_CLUE (flow1)->halign, HTML_CLUE (flow2)->halign))) {
 		HTMLCursor *dest, *source;
 
-		if (html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
-			dest = html_cursor_dup (e->cursor);
-			source = html_cursor_dup (e->mark);
-		} else {
-			source = html_cursor_dup (e->cursor);
-			dest = html_cursor_dup (e->mark);
-		}
+		dest = html_cursor_dup (e->cursor);
+		source = html_cursor_dup (e->mark);
 
 		html_engine_selection_push (e);
 		html_engine_disable_selection (e);
@@ -524,9 +556,9 @@ delete_object_do (HTMLEngine *e, HTMLObject **object, guint *len, HTMLUndoDirect
 	html_engine_freeze (e);
 	level = prepare_delete_bounds (e, &from, &to, &left, &right);
 	place_cursor_before_mark (e);
-	move_cursor_before_delete (e);
 	if (add_undo)
 		check_flows (e, dir);
+	move_cursor_before_delete (e);
 	html_engine_disable_selection (e);
 	*len     = 0;
 	*object  = html_object_op_cut  (HTML_OBJECT (from->data), e, from->next, to->next, left, right, len);
@@ -546,6 +578,8 @@ check_table_0 (HTMLEngine *e)
 
 	tail = e->mark->position < e->cursor->position ? e->cursor : e->mark;
 
+	if (html_cursor_backward (tail, e) && (!HTML_IS_TABLE (tail->object) || tail->offset))
+		html_cursor_forward (tail, e);
 	while (tail->offset == 0 && HTML_IS_TABLE (tail->object) && e->mark->position != e->cursor->position)
 		html_cursor_backward (tail, e);
 }
@@ -557,6 +591,8 @@ check_table_1 (HTMLEngine *e)
 
 	head = e->mark->position > e->cursor->position ? e->cursor : e->mark;
 
+	if (html_cursor_forward (head, e) && (!HTML_IS_TABLE (head->object) || head->offset == 0))
+		html_cursor_backward (head, e);
 	while (head->offset == 1 && HTML_IS_TABLE (head->object) && e->mark->position != e->cursor->position)
 		html_cursor_forward (head, e);
 }
@@ -597,24 +633,109 @@ validate_tables (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo, gboole
 	return FALSE;
 }
 
+static inline gboolean
+in_aligned (HTMLCursor *cursor)
+{
+	return cursor->object->parent && HTML_IS_CLUEALIGNED (cursor->object->parent);
+}
+
+struct _FixEmptyAlignedUndo {
+	HTMLUndoData data;
+
+	HTMLObject *ac;
+};
+typedef struct _FixEmptyAlignedUndo FixEmptyAlignedUndo;
+
+static void
+fix_empty_aligned_undo_destroy (HTMLUndoData *data)
+{
+	FixEmptyAlignedUndo *undo = (FixEmptyAlignedUndo *) data;
+
+	if (undo->ac)
+		html_object_destroy (undo->ac);
+}
+
+static void
+fix_empty_aligned_undo_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, guint position_after)
+{
+	HTMLObject *ac, *flow;
+
+	g_return_if_fail (html_object_is_text (e->cursor->object) && HTML_TEXT (e->cursor->object)->text_len == 0
+			  && e->cursor->object->parent && HTML_IS_CLUEFLOW (e->cursor->object->parent));
+
+	ac = ((FixEmptyAlignedUndo *) data)->ac;
+	((FixEmptyAlignedUndo *) data)->ac = NULL;
+
+	html_engine_freeze (e);
+	flow = e->cursor->object->parent;
+	html_clue_remove_text_slaves (HTML_CLUE (flow));
+	html_clue_append_after (HTML_CLUE (flow), ac, e->cursor->object);
+	html_object_remove_child (flow, e->cursor->object);
+	html_clue_append (HTML_CLUE (ac), e->cursor->object);
+	html_object_change_set_down (flow, HTML_CHANGE_ALL);
+	html_engine_thaw (e);
+}
+
+static void
+fix_empty_aligned_setup_undo (HTMLEngine *e, HTMLUndoDirection dir, HTMLObject *ac)
+{
+	FixEmptyAlignedUndo *undo;
+
+	undo = g_new (FixEmptyAlignedUndo, 1);
+
+	html_undo_data_init (HTML_UNDO_DATA (undo));
+	undo->data.destroy = fix_empty_aligned_undo_destroy;
+	undo->ac       = ac;
+
+	html_undo_add_action (e->undo,
+			      html_undo_action_new ("Remove empty aligned", fix_empty_aligned_undo_action,
+						    HTML_UNDO_DATA (undo), html_cursor_get_position (e->cursor),
+						    html_cursor_get_position (e->cursor)),
+			      dir);
+}
+
+static void
+fix_empty_aligned (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
+{
+	if (html_object_is_text (e->cursor->object) && e->cursor->object->parent && HTML_IS_CLUEALIGNED (e->cursor->object->parent)) {
+		HTMLObject *ac = e->cursor->object->parent;
+
+		if (ac->parent && HTML_IS_CLUEFLOW (ac->parent)) {
+			html_engine_freeze (e);
+			html_clue_remove_text_slaves (HTML_CLUE (ac));
+			html_object_remove_child (ac, e->cursor->object);
+			html_clue_append_after (HTML_CLUE (ac->parent), e->cursor->object, ac);
+			html_object_change_set_down (ac->parent, HTML_CHANGE_ALL);
+			html_object_remove_child (ac->parent, ac);
+			if (add_undo)
+				fix_empty_aligned_setup_undo (e, dir, ac);
+			html_engine_thaw (e);
+		}
+	}
+}
+
 static gint
 delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoDirection dir, gboolean add_undo)
 {
 	html_engine_edit_selection_updater_update_now (e->selection_updater);
 	if (html_engine_is_selection_active (e)) {
 		HTMLObject *object;
-		guint len, position_before, saved_position;
+		guint len, position_before, saved_position, end_position;
 		gint level;
 		gboolean backward;
 		gboolean fix_para;
 
-		if (!html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))
-		    && !html_clueflow_is_empty (HTML_CLUEFLOW (e->mark->object->parent))) {
+		end_position = MIN (e->cursor->position, e->mark->position);
+		if (HTML_IS_TABLE (e->cursor->object)
+		    || (e->cursor->object->parent && e->cursor->object->parent->parent && HTML_IS_TABLE_CELL (e->cursor->object->parent->parent))
+		    || HTML_IS_TABLE (e->mark->object)
+		    || (e->mark->object->parent && e->mark->object->parent->parent && HTML_IS_TABLE_CELL (e->mark->object->parent->parent))) {
 			check_table_0 (e);
 			check_table_1 (e);
 		}
 		if (e->cursor->position == e->mark->position) {
 			html_engine_disable_selection (e);
+			html_cursor_jump_to_position (e->cursor, e, end_position);
 			return 0;
 		}
 
@@ -631,6 +752,7 @@ delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoD
 			insert_setup_undo (e, 1, position_before, dir, FALSE, FALSE);
 			e->cursor->position = saved_position;
 		}
+		level = html_object_get_parent_level (e->cursor->object) - level + 1;
 		if (add_undo) {
 			delete_setup_undo (e, object, len, position_before + (backward ? 1 : 0), level, dir);
 		} else
@@ -639,6 +761,7 @@ delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoD
 		if (backward)
 			html_cursor_backward (e->cursor, e);
 		gtk_html_editor_event (e->widget, GTK_HTML_EDITOR_EVENT_DELETE, NULL);
+		fix_empty_aligned (e, dir, add_undo);
 
 		return level;
 	}
@@ -646,14 +769,6 @@ delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoD
 	return 0;
 }
 
-void
-html_engine_delete (HTMLEngine *e)
-{
-	html_undo_level_begin (e->undo, "Delete", "Undelete");
-	delete_object (e, NULL, NULL, HTML_UNDO_UNDO, TRUE);
-	html_undo_level_end (e->undo);
-}
-
 gint
 html_engine_cut (HTMLEngine *e)
 {
@@ -829,8 +944,127 @@ insert_setup_undo (HTMLEngine *e, guint len, guint position_before, HTMLUndoDire
 			      dir);
 }
 
+static gboolean fix_aligned_position (HTMLEngine *e, guint *position_after, HTMLUndoDirection dir);
+
 static void
-insert_object (HTMLEngine *e, HTMLObject *obj, guint len, guint position_after, gint level,
+fix_aligned_redo_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, guint position_after)
+{
+	guint pa;
+
+	fix_aligned_position (e, &pa, html_undo_direction_reverse (dir));
+}
+
+static void
+fix_aligned_undo_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, guint position_after)
+{
+	HTMLObject *cf = e->cursor->object->parent;
+	HTMLUndoData *undo;
+	guint position_before = e->cursor->position;
+
+	undo = g_new (HTMLUndoData, 1);
+
+	if (!html_cursor_forward (e->cursor, e))
+		g_assert (html_cursor_backward (e->cursor, e));
+	else
+		e->cursor->position --;
+
+	html_clue_remove (HTML_CLUE (cf->parent), cf);
+	html_object_destroy (cf);
+
+	html_undo_add_action (e->undo,
+			      html_undo_action_new ("Fix aligned", fix_aligned_redo_action,
+						    undo, html_cursor_get_position (e->cursor),
+						    position_before),
+			      html_undo_direction_reverse (dir));
+}
+
+static void
+fix_align_setup_undo (HTMLEngine *e, guint position_before, HTMLUndoDirection dir)
+{
+	HTMLUndoData *undo;
+
+	undo = g_new (HTMLUndoData, 1);
+
+	html_undo_data_init (HTML_UNDO_DATA (undo));
+	/* printf ("insert undo len %d\n", len); */
+
+	html_undo_add_action (e->undo,
+			      html_undo_action_new ("Undo aligned fix", fix_aligned_undo_action,
+						    undo, html_cursor_get_position (e->cursor),
+						    position_before),
+			      dir);
+}
+
+static gboolean
+fix_aligned_position (HTMLEngine *e, guint *position_after, HTMLUndoDirection dir)
+{
+	gboolean rv = FALSE;
+	if (in_aligned (e->cursor)) {
+		/* printf ("in aligned\n"); */
+		if (e->cursor->offset) {
+			if (html_cursor_forward (e->cursor, e))
+				(*position_after) ++;
+			if (in_aligned (e->cursor)) {
+				HTMLObject *cf;
+				HTMLObject *cluev;
+				HTMLObject *flow;
+
+				/* printf ("aligned: needs fixing\n"); */
+				html_engine_freeze (e);
+				cf = html_clueflow_new_from_flow (HTML_CLUEFLOW (e->cursor->object->parent->parent));
+				flow = e->cursor->object->parent->parent;
+				cluev = flow->parent;
+				e->cursor->object = html_engine_new_text_empty (e);
+				html_clue_append (HTML_CLUE (cf), e->cursor->object);
+				html_clue_append_after (HTML_CLUE (cluev), cf, flow);
+				e->cursor->offset = 0;
+				e->cursor->position ++;
+				(*position_after) ++;
+#ifdef OP_DEBUG
+				gtk_html_debug_dump_tree_simple (e->clue, 0);
+#endif
+				fix_align_setup_undo (e, e->cursor->position, dir);
+				html_engine_thaw (e);
+				rv = TRUE;
+				if (e->cursor->object->parent && HTML_IS_CLUEALIGNED (e->cursor->object->parent))
+					html_cursor_forward (e->cursor, e);
+
+			}
+		} else {
+			if (html_cursor_backward (e->cursor, e))
+				(*position_after) --;
+			if (in_aligned (e->cursor)) {
+				HTMLObject *cf;
+				HTMLObject *cluev;
+				HTMLObject *flow;
+
+				/* printf ("aligned: needs fixing\n"); */
+				html_engine_freeze (e);
+				cf = html_clueflow_new_from_flow (HTML_CLUEFLOW (e->cursor->object->parent->parent));
+				flow = e->cursor->object->parent->parent;
+				cluev = flow->parent;
+				e->cursor->object = html_engine_new_text_empty (e);
+				html_clue_append (HTML_CLUE (cf), e->cursor->object);
+				if (flow->prev)
+					html_clue_append_after (HTML_CLUE (cluev), cf, flow->prev);
+				else
+					html_clue_prepend (HTML_CLUE (cluev), cf);
+				e->cursor->offset = 0;
+#ifdef OP_DEBUG
+				gtk_html_debug_dump_tree_simple (e->clue, 0);
+#endif
+				fix_align_setup_undo (e, e->cursor->position, dir);
+				html_engine_thaw (e);
+				rv = TRUE;
+			}
+		}
+	}
+
+	return rv;
+}
+
+static void
+insert_object_for_undo (HTMLEngine *e, HTMLObject *obj, guint len, guint position_after, gint level,
 	       HTMLUndoDirection dir, gboolean check)
 {
 	gboolean delete_paragraph_before = FALSE;
@@ -845,6 +1079,14 @@ insert_object (HTMLEngine *e, HTMLObject *obj, guint len, guint position_after,
 			   dir, delete_paragraph_before, delete_paragraph_after);
 }
 
+static void
+insert_object (HTMLEngine *e, HTMLObject *obj, guint len, guint position_after, gint level,
+	       HTMLUndoDirection dir, gboolean check)
+{
+	fix_aligned_position (e, &position_after, dir);
+	insert_object_for_undo (e, obj, len, position_after, level, dir, check);
+}
+
 void
 html_engine_insert_object (HTMLEngine *e, HTMLObject *o, guint len, gint level)
 {
@@ -887,8 +1129,14 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
 	GList *left=NULL, *right=NULL;
 	HTMLCursor *orig;
 	guint position_before;
+	guint position_after;
+
+	if (dir == HTML_UNDO_UNDO)
+		if (fix_aligned_position (e, &position_after, dir))
+			return;
 
 	html_engine_freeze (e);
+
 	position_before = e->cursor->position;
 	orig = html_cursor_dup (e->cursor);
 	split_and_add_empty_texts (e, 2, &left, &right);
@@ -1037,7 +1285,7 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 	gtk_html_editor_event_command (e->widget, GTK_HTML_COMMAND_INSERT_PARAGRAPH, TRUE);
 
 	do {
-		nl   = g_utf8_strchr (text, '\n');
+		nl   = g_utf8_strchr (text, -1, '\n');
 		alen = nl ? g_utf8_pointer_to_offset (text, nl) : len;
 		if (alen) {
 			HTMLObject *o;
@@ -1061,7 +1309,7 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 			}
 			insert_object (e, o, html_object_get_length (o), e->cursor->position + html_object_get_length (o),
 				       1, HTML_UNDO_UNDO, check);
-			if (alen == 1 && !HTML_IS_PLAIN_PAINTER (e->painter) && GTK_HTML_PROPERTY (e->widget, magic_smileys))
+			if (alen == 1 && !HTML_IS_PLAIN_PAINTER (e->painter))
 				use_pictograms (e);
 		}
 		if (nl) {
@@ -1380,3 +1628,74 @@ html_engine_cut_and_paste (HTMLEngine *e, const gchar *undo_op_name, const gchar
 		html_object_forall (e->clipboard, e, iterator, data);
 	html_engine_cut_and_paste_end (e);
 }
+
+static void
+delete_upto (HTMLEngine *e, HTMLCursor **start, HTMLCursor **end, HTMLObject *object, guint offset)
+{
+	guint position;
+
+	html_cursor_destroy (e->mark);
+	e->mark = *start;
+	html_cursor_jump_to (e->cursor, e, object, offset);
+	position = e->cursor->position;
+	delete_object (e, NULL, NULL, HTML_UNDO_UNDO, TRUE);
+	*start = html_cursor_dup (e->cursor);
+	html_cursor_forward (*start, e);
+	(*end)->position -= position - e->cursor->position;
+}
+
+void
+html_engine_delete (HTMLEngine *e)
+{
+	html_undo_level_begin (e->undo, "Delete", "Undelete");
+	html_engine_edit_selection_updater_update_now (e->selection_updater);
+	if (html_engine_is_selection_active (e)) {
+		HTMLCursor *start = html_cursor_dup (e->mark->position < e->cursor->position ? e->mark : e->cursor);
+		HTMLCursor *end = html_cursor_dup (e->mark->position < e->cursor->position ? e->cursor : e->mark);
+
+		while (start->position < end->position) {
+			if (start->object->parent->parent == end->object->parent->parent) {
+				if (e->mark)
+					html_cursor_destroy (e->mark);
+				html_cursor_destroy (e->cursor);
+				e->mark = start;
+				e->cursor = end;
+				start = end = NULL;
+				delete_object (e, NULL, NULL, HTML_UNDO_UNDO, TRUE);
+				break;
+			} else {
+				HTMLObject *prev = NULL, *cur = start->object;
+
+				/* go thru current cluev */
+				do {
+				        /* go thru current flow */
+					while (cur) {
+				                /* lets look if container is whole contained in the selection */
+						if (html_object_is_container (cur)) {
+							html_cursor_jump_to (e->cursor, e, cur, html_object_get_length (cur));
+							if (e->cursor->position > end->position) {
+								/* it's not => delete upto this container */
+
+								delete_upto (e, &start, &end, cur, 0);
+								prev = NULL;
+								break;
+							}
+						}
+						prev = cur;
+						cur = html_object_next_not_slave (cur);
+					}
+				} while (prev && prev->parent->next && (cur = html_object_head (prev->parent->next)));
+
+				if (prev)
+				        /* cluev end is in the selection */
+					delete_upto (e, &start, &end, prev, html_object_get_length (prev));
+			}
+		}
+
+		if (start)
+			html_cursor_destroy (start);
+		if (end)
+			html_cursor_destroy (end);
+	}
+	html_undo_level_end (e->undo);
+}
