@@ -178,7 +178,7 @@ html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end
 
 	e->need_spell_check = FALSE;
 
-	if (!e->widget->editor_api || !GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check
+	if (!e->widget->editor_api || !GTK_HTML_CLASS (GTK_WIDGET_GET_CLASS (e->widget))->properties->live_spell_check
 	    || !begin->object->parent)
 		return;
 
@@ -704,5 +704,5 @@ html_engine_set_title (HTMLEngine *e, const gchar *title)
 	if (e->title)
 		g_string_free (e->title, TRUE);
 	e->title = g_string_new (title);
-	gtk_signal_emit_by_name (GTK_OBJECT (e), "title_changed");
+	g_signal_emit_by_name (e, "title_changed");
 }
