@@ -21,7 +21,6 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include "gtkhtml.h"
@@ -40,7 +39,7 @@ print_header_footer (HTMLPainter *painter, HTMLEngine *engine, gint width, gint
 		     GtkHTMLPrintCallback cb, gpointer user_data)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *context = printer->print_context;
+	GnomePrintContext *context = printer->context;
 	gdouble gx, gy;
 
 	gnome_print_gsave (context);
@@ -64,11 +63,11 @@ print_page (HTMLPainter *painter,
 	    GtkHTMLPrintCallback header_print, GtkHTMLPrintCallback footer_print, gpointer user_data)
 {
 	HTMLPrinter *printer = HTML_PRINTER (painter);
-	GnomePrintContext *context = printer->print_context;
+	GnomePrintContext *context = printer->context;
 
 	html_painter_begin (painter, 0, 0, page_width, page_height);
 	if (header_print)
-		print_header_footer (painter, engine, page_width, 0, header_height/printer->scale, header_print, user_data);
+		print_header_footer (painter, engine, page_width, 0, header_height, header_print, user_data);
 	gnome_print_gsave (context);
 	html_painter_set_clip_rectangle (painter, 0, header_height, page_width, body_height);
 #ifdef CLIP_DEBUG
@@ -77,12 +76,11 @@ print_page (HTMLPainter *painter,
 	html_object_draw (engine->clue, painter,
 			  0, start_y,
 			  page_width, body_height,
-			  0, -start_y + header_height / printer->scale);
+			  0, -start_y + header_height);
 	gnome_print_grestore (context);
 	if (footer_print)
-		print_header_footer (painter, engine, page_width*printer->scale,
-				     page_height - SCALE_GNOME_PRINT_TO_ENGINE (footer_height/printer->scale),
-				     footer_height/printer->scale, footer_print, user_data);
+		print_header_footer (painter, engine, page_width, page_height - SCALE_GNOME_PRINT_TO_ENGINE (footer_height),
+				     footer_height, footer_print, user_data);
 	html_painter_end (painter);
 }
 
@@ -101,13 +99,12 @@ print_all_pages (HTMLPainter *painter,
 	page_height = html_printer_get_page_height (printer);
 	page_width  = html_printer_get_page_width (printer);
 
-	if (header_height + footer_height >= page_height*printer->scale) {
+	if (header_height + footer_height >= page_height) {
 		header_print = footer_print = NULL;
 		g_warning ("Page header height + footer height >= page height, disabling header/footer printing");
 	}
 
-	body_height = page_height
-		- SCALE_GNOME_PRINT_TO_ENGINE (header_height/printer->scale + footer_height/printer->scale);
+	body_height = page_height - SCALE_GNOME_PRINT_TO_ENGINE (header_height + footer_height);
 	split_offset = 0;
 
 	document_height = html_engine_get_doc_height (engine);
@@ -158,13 +155,13 @@ print_with_header_footer (HTMLEngine *engine,
 {
 	HTMLPainter *printer;
 	HTMLPainter *old_painter;
-	GtkHTMLClassProperties *prop = GTK_HTML_CLASS (GTK_OBJECT (engine->widget)->klass)->properties;
+	GtkHTMLClassProperties *prop = GTK_HTML_CLASS (GTK_WIDGET_GET_CLASS (engine->widget))->properties;
 	gint pages = 0;
 
 	g_return_val_if_fail (engine->clue != NULL, 0);
 
 	printer = html_printer_new (print_context, GTK_HTML (engine->widget)->priv->print_master);
-	html_font_manager_set_default (html_engine_font_manager_with_painter (engine, printer),
+	html_font_manager_set_default (&printer->font_manager,
 				       prop->font_var_print,      prop->font_fix_print,
 				       prop->font_var_size_print, prop->font_var_print_points,
 				       prop->font_fix_size_print, prop->font_fix_print_points);
@@ -174,7 +171,7 @@ print_with_header_footer (HTMLEngine *engine,
 
 		old_painter = engine->painter;
 
-		gtk_object_ref (GTK_OBJECT (old_painter));
+		g_object_ref (old_painter);
 		html_engine_set_painter (engine, printer);
 
 		min_width = html_engine_calc_min_width (engine);
@@ -182,6 +179,7 @@ print_with_header_footer (HTMLEngine *engine,
 		/* printf ("min_width %d page %d\n", min_width, page_width); */
 		if (min_width > page_width) {
 			HTML_PRINTER (printer)->scale = MAX (0.5, ((gdouble) page_width) / min_width);
+			html_font_manager_clear_font_cache (&printer->font_manager);
 			html_object_change_set_down (engine->clue, HTML_CHANGE_ALL);
 			html_engine_calc_size (engine, NULL);
 			/* printf ("scale %lf\n", HTML_PRINTER (printer)->scale);
@@ -194,12 +192,13 @@ print_with_header_footer (HTMLEngine *engine,
 					 user_data, do_print);
 
 		html_engine_set_painter (engine, old_painter);
-		gtk_object_unref (GTK_OBJECT (old_painter));
+		g_object_unref (G_OBJECT (old_painter));
 	} else {
-		gnome_ok_dialog (_("Cannot allocate default font for printing\n"));
+		/* TODO2 dialog instead of warning */
+		g_warning (_("Cannot allocate default font for printing\n"));
 	}
 
-	gtk_object_unref (GTK_OBJECT (printer));
+	g_object_unref (G_OBJECT (printer));
 
 	return pages;
 }
