@@ -35,8 +35,8 @@
 #include "htmlsearch.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
-#include <libgnome/gnome-i18n.h>
 
+#define USE_SCROLLED_WINDOW 1
 #ifndef USE_SCROLLED_WINDOW
 #include <gal/widgets/e-scroll-frame.h>
 #endif
@@ -51,8 +51,7 @@ frame_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpoi
 	HTMLFrame *frame = HTML_FRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
 
-	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested",
-				 url, handle);
+	g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
 }
 
 static void
@@ -75,7 +74,7 @@ frame_submit (GtkHTML *html,
 	HTMLFrame *frame = HTML_FRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
 
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "submit", method, action, encoding);
+	g_signal_emit_by_name (parent, "submit", method, action, encoding);
 }
 
 static void
@@ -95,7 +94,7 @@ frame_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 	gboolean ret_val;
 
 	ret_val = FALSE;
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "object_requested", eb, &ret_val);
+	g_signal_emit_by_name (parent, "object_requested", eb, &ret_val);
 	return ret_val;
 }
 
@@ -103,10 +102,10 @@ static void
 frame_set_gdk_painter (HTMLFrame *frame, HTMLPainter *painter)
 {
 	if (painter)
-		gtk_object_ref (GTK_OBJECT (painter));
+		g_object_ref (G_OBJECT (painter));
 	
 	if (frame->gdk_painter)
-		gtk_object_unref (GTK_OBJECT (frame->gdk_painter));
+		g_object_unref (G_OBJECT (frame->gdk_painter));
 
 	frame->gdk_painter = painter;
 }
@@ -193,12 +192,12 @@ draw (HTMLObject *o,
 {
 	HTMLFrame   *frame  = HTML_FRAME (o);
 	HTMLEngine   *e       = GTK_HTML (frame->html)->engine;
-	ArtIRect paint;
+	GdkRectangle paint;
 
 	if (GTK_OBJECT_TYPE (e->painter) == HTML_TYPE_PRINTER) {
 		gint pixel_size = html_painter_get_pixel_size (e->painter);
-		html_object_calc_intersection (o, &paint, x, y, width, height);
-		if (art_irect_empty (&paint))
+
+		if (!html_object_intersect (o, &paint, x, y, width, height))
 			return;
 
 		html_object_draw (e->clue, e->painter,
@@ -216,7 +215,7 @@ set_painter (HTMLObject *o, HTMLPainter *painter)
 	HTMLFrame *frame;
 
 	frame = HTML_FRAME (o);
-	if (GTK_OBJECT_TYPE (GTK_HTML (frame->html)->engine->painter) != HTML_TYPE_PRINTER) {
+	if (G_OBJECT_TYPE (GTK_HTML (frame->html)->engine->painter) != HTML_TYPE_PRINTER) {
 		frame_set_gdk_painter (frame, GTK_HTML (frame->html)->engine->painter);
 	}
 	
@@ -265,7 +264,7 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 		height = html_engine_get_doc_height (e);
 		width = html_engine_get_doc_width (e);
 
-		gtk_widget_set_usize (frame->scroll, width, height);
+		gtk_widget_set_size_request (frame->scroll, width, height);
 		gtk_widget_queue_resize (frame->scroll);
 		
 		html_frame_set_scrolling (frame, GTK_POLICY_NEVER);
@@ -401,7 +400,7 @@ destroy (HTMLObject *o)
 	frame_set_gdk_painter (frame, NULL);
 
 	if (frame->html) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (frame->html), o);
+		g_signal_handlers_disconnect_matched (frame->html, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, o);
 		frame->html = NULL;
 	}
 	g_free ((frame)->url);
@@ -434,7 +433,7 @@ html_frame_set_margin_height (HTMLFrame *frame, gint margin_height)
 void
 html_frame_set_scrolling (HTMLFrame *frame, GtkPolicyType scroll)
 {
-#if E_USE_SCROLLED_WINDOW
+#if USE_SCROLLED_WINDOW
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (frame->scroll),
 					scroll, scroll);
 #else
@@ -454,7 +453,7 @@ html_frame_set_size (HTMLFrame *frame, gint width, gint height)
 	if (height > 0)
 		frame->height = height;
 	
-	gtk_widget_set_usize (frame->scroll, width, height);
+	gtk_widget_set_size_request (frame->scroll, width, height);
 }
 
 void 
@@ -496,7 +495,7 @@ html_frame_init (HTMLFrame *frame,
 	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
 	html_engine_set_tokenizer (new_html->engine, new_tokenizer);
-	gtk_object_unref (GTK_OBJECT (new_tokenizer));
+	g_object_unref (G_OBJECT (new_tokenizer));
 	new_tokenizer = NULL;
 
 	gtk_html_set_default_content_type (new_html,
@@ -518,9 +517,7 @@ html_frame_init (HTMLFrame *frame,
 	new_html->engine->clue->parent = HTML_OBJECT (frame);
 
 
-	gtk_signal_connect (GTK_OBJECT (new_html), "url_requested",
-			    GTK_SIGNAL_FUNC (frame_url_requested),
-			    (gpointer)frame);
+	g_signal_connect (new_html, "url_requested", G_CALLBACK (frame_url_requested), frame);
 #if 0
 	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
 	 * on_url and link_clicked are emitted from the toplevel widget not
@@ -533,18 +530,10 @@ html_frame_init (HTMLFrame *frame,
 			    GTK_SIGNAL_FUNC (frame_link_clicked),
 			    (gpointer)frame);	
 #endif
-	gtk_signal_connect (GTK_OBJECT (new_html), "size_changed",
-			    GTK_SIGNAL_FUNC (frame_size_changed),
-			    (gpointer)frame);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "object_requested",
-			    GTK_SIGNAL_FUNC (frame_object_requested),
-			    (gpointer)frame);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "submit",
-			    GTK_SIGNAL_FUNC (frame_submit),
-			    (gpointer)frame);
-	gtk_signal_connect (GTK_OBJECT (new_html), "set_base",
-			    GTK_SIGNAL_FUNC (frame_set_base), 
-			    (gpointer)frame);
+	g_signal_connect (new_html, "size_changed", G_CALLBACK (frame_size_changed), frame);	
+	g_signal_connect (new_html, "object_requested", G_CALLBACK (frame_object_requested), frame);	
+	g_signal_connect (new_html, "submit", G_CALLBACK (frame_submit), frame);
+	g_signal_connect (new_html, "set_base", G_CALLBACK (frame_set_base), frame);
 
 	html_frame_set_margin_height (frame, 0);
 	html_frame_set_margin_width (frame, 0);
@@ -554,15 +543,9 @@ html_frame_init (HTMLFrame *frame,
 	  GTK_SIGNAL_FUNC (frame_button_press_event), frame);
 	*/
 
-	if (depth < 10) {
-		gtk_signal_emit_by_name (GTK_OBJECT (parent_html->engine), 
-					 "url_requested", src, handle);
-	} else {
-		gtk_html_stream_printf (handle, "%s", _("Error: maximium frame depth exceded"));
-		gtk_html_stream_close (handle, GTK_HTML_STREAM_OK);
-	}
+	g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
 
-	gtk_widget_set_usize (scrolled_window, width, height);
+	gtk_widget_set_size_request (scrolled_window, width, height);
 
 	gtk_widget_show (scrolled_window);	
 	frame->scroll = scrolled_window;
@@ -570,8 +553,7 @@ html_frame_init (HTMLFrame *frame,
 
 	html_embedded_set_widget (em, scrolled_window);
 
-	gtk_signal_connect(GTK_OBJECT (scrolled_window), "button_press_event",
-			   GTK_SIGNAL_FUNC (html_frame_grab_cursor), NULL);
+	g_signal_connect (scrolled_window, "button_press_event", G_CALLBACK (html_frame_grab_cursor), NULL);
 
 	/* inherit the current colors from our parent */
 	html_colorset_set_unchanged (new_html->engine->defaultSettings->color_set,
@@ -580,14 +562,10 @@ html_frame_init (HTMLFrame *frame,
 				     parent_html->engine->settings->color_set);
 	html_painter_set_focus (new_html->engine->painter, parent_html->engine->have_focus);
 	/*
-	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
-			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
-			    GTK_SIGNAL_FUNC (on_button_press_event), popup_menu);
-	gtk_signal_connect (GTK_OBJECT (html), "redirect",
-			    GTK_SIGNAL_FUNC (on_redirect), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
-			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
+	g_signal_connect (html, "title_changed", G_CALLBACK (title_changed_cb), app);
+	g_signal_connect (html, "button_press_event", G_CALLBACK (on_button_press_event), popup_menu);
+	g_signal_connect (html, "redirect", G_CALLBACK (on_redirect), NULL);
+	g_signal_connect (html, "object_requested", G_CALLBACK (object_requested_cmd), NULL);
 	*/
 }
 
