@@ -26,50 +26,44 @@
 #include <stdlib.h>
 #include "gtkhtml-compat.h"
 
-#include <gal/unicode/gunicode.h>
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
-#include "htmlengine.h"
 #include "htmlentity.h"
 #include "htmlpainter.h"
-#include "htmlgdkpainter.h"
-#include "htmlplainpainter.h"
-#include "htmlprinter.h"
 
 
 /* Convenience macro to extract the HTMLPainterClass from a GTK+ object.  */
 #define HP_CLASS(obj)					\
-	HTML_PAINTER_CLASS (GTK_OBJECT (obj)->klass)
+	HTML_PAINTER_CLASS (G_OBJECT_GET_CLASS (obj))
 
 /* Our parent class.  */
-static GtkObjectClass *parent_class = NULL;
-
-static gint calc_text_bytes_delta (const gchar *text, gint len, gint line_offset, gint *translated_len, gboolean tabs);
-static gint translate_text_special_chars (const gchar *text, gchar *translated, gint len, gint line_offset, gboolean tabs);
+static GObjectClass *parent_class = NULL;
 
 
-/* GtkObject methods.  */
+/* GObject methods.  */
 
 static void
-finalize (GtkObject *object)
+finalize (GObject *object)
 {
 	HTMLPainter *painter;
 
 	painter = HTML_PAINTER (object);
+	html_font_manager_finalize (&painter->font_manager);
+
 	html_colorset_destroy (painter->color_set);
 
 	/* FIXME ownership of the color set?  */
 
-	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 
 #define DEFINE_UNIMPLEMENTED(method)						\
 	static gint								\
-	method##_unimplemented (GtkObject *obj)					\
+	method##_unimplemented (GObject *obj)					\
 	{									\
 		g_warning ("Class `%s' does not implement `" #method "'\n",	\
-			   gtk_type_name (GTK_OBJECT_TYPE (obj)));		\
+			   g_type_name (G_TYPE_FROM_INSTANCE (obj)));		\
 		return 0;							\
 	}
 
@@ -83,10 +77,8 @@ DEFINE_UNIMPLEMENTED (unref_font)
 DEFINE_UNIMPLEMENTED (alloc_color)
 DEFINE_UNIMPLEMENTED (free_color)
 
-DEFINE_UNIMPLEMENTED (calc_ascent)
-DEFINE_UNIMPLEMENTED (calc_descent)
-DEFINE_UNIMPLEMENTED (calc_text_width)
-DEFINE_UNIMPLEMENTED (calc_text_width_bytes)
+DEFINE_UNIMPLEMENTED (calc_text_size)
+DEFINE_UNIMPLEMENTED (calc_text_size_bytes)
 
 DEFINE_UNIMPLEMENTED (set_pen)
 DEFINE_UNIMPLEMENTED (get_black)
@@ -110,29 +102,29 @@ DEFINE_UNIMPLEMENTED (get_pixel_size)
 DEFINE_UNIMPLEMENTED (get_page_width)
 DEFINE_UNIMPLEMENTED (get_page_height)
 
-DEFINE_UNIMPLEMENTED (get_font_manager_id)
-
 
 static void
-init (GtkObject *object, HTMLPainterClass *real_klass)
+html_painter_init (GObject *object, HTMLPainterClass *real_klass)
 {
 	HTMLPainter *painter;
 
 	painter = HTML_PAINTER (object);
 	painter->color_set = html_colorset_new (NULL);
 
+	html_font_manager_init (&painter->font_manager, painter);
 	painter->font_style = GTK_HTML_FONT_STYLE_DEFAULT;
 	painter->font_face = NULL;
 }
 
 static void
-class_init (GtkObjectClass *object_class)
+html_painter_class_init (GObjectClass *object_class)
 {
 	HTMLPainterClass *class;
 
 	class = HTML_PAINTER_CLASS (object_class);
 
 	object_class->finalize = finalize;
+	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
 	class->begin = (gpointer) begin_unimplemented;
 	class->end = (gpointer) end_unimplemented;
@@ -144,10 +136,8 @@ class_init (GtkObjectClass *object_class)
 	class->alloc_color = (gpointer) alloc_color_unimplemented;
 	class->free_color = (gpointer) free_color_unimplemented;
 
-	class->calc_ascent = (gpointer) calc_ascent_unimplemented;
-	class->calc_descent = (gpointer) calc_descent_unimplemented;
-	class->calc_text_width = (gpointer) calc_text_width_unimplemented;
-	class->calc_text_width_bytes = (gpointer) calc_text_width_bytes_unimplemented;
+	class->calc_text_size = (gpointer) calc_text_size_unimplemented;
+	class->calc_text_size_bytes = (gpointer) calc_text_size_bytes_unimplemented;
 
 	class->set_pen = (gpointer) set_pen_unimplemented;
 	class->get_black = (gpointer) get_black_unimplemented;
@@ -171,40 +161,35 @@ class_init (GtkObjectClass *object_class)
 
 	class->get_page_width  = (gpointer) get_page_width_unimplemented;
 	class->get_page_height = (gpointer) get_page_height_unimplemented;
-
-	class->get_font_manager_id = (gpointer) get_font_manager_id_unimplemented;
-
-	parent_class = gtk_type_class (gtk_object_get_type ());
 }
 
-
-GtkType
+GType
 html_painter_get_type (void)
 {
-	static GtkType type = 0;
+	static GType html_painter_type = 0;
 
-	if (type == 0) {
-		static const GtkTypeInfo info = {
-			"HTMLPainter",
-			sizeof (HTMLPainter),
+	if (html_painter_type == 0) {
+		static const GTypeInfo html_painter_info = {
 			sizeof (HTMLPainterClass),
-			(GtkClassInitFunc) class_init,
-			(GtkObjectInitFunc) init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
+			NULL,
+			NULL,
+			(GClassInitFunc) html_painter_class_init,
+			NULL,
+			NULL,
+			sizeof (HTMLPainter),
+			1,
+			(GInstanceInitFunc) html_painter_init,
 		};
-
-		type = gtk_type_unique (GTK_TYPE_OBJECT, &info);
+		html_painter_type = g_type_register_static (G_TYPE_OBJECT, "HTMLPainter", &html_painter_info, 0);
 	}
 
-	return type;
+	return html_painter_type;
 }
 
 HTMLPainter *
 html_painter_new (void)
 {
-	return gtk_type_new (html_painter_get_type ());
+	return g_object_new (HTML_TYPE_PAINTER, NULL);
 }
 
 
@@ -289,76 +274,29 @@ html_painter_set_font_face (HTMLPainter *painter,
 	}
 }
 
-static HTMLFont *
-get_html_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
-{
-	HTMLEngineClass *ec = gtk_type_class (html_engine_get_type ());
-	HTMLFontManager *fm = &ec->font_manager [html_painter_get_font_manager_id (painter)];
-
-	return html_font_manager_get_font (fm, face, style);
-}
-
-HTMLFont *
-html_painter_get_html_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
-{
-	return get_html_font (painter, face, style);
-}
-
 gpointer
 html_painter_get_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
 {
-	HTMLFont *font = get_html_font (painter, face, style);
+	HTMLFont *font;
 
+	font = html_font_manager_get_font (&painter->font_manager, face, style);
 	return font ? font->data : NULL;
 }
 
-guint
-html_painter_calc_ascent (HTMLPainter *painter,
-			  GtkHTMLFontStyle font_style,
-			  HTMLFontFace *face)
+void
+html_painter_calc_text_size (HTMLPainter *painter,
+			     const gchar *text,
+			     guint len, GList *items, GList *glyphs, gint *line_offset,
+			     GtkHTMLFontStyle font_style,
+			     HTMLFontFace *face,
+			     gint *width, gint *asc, gint *dsc)
 {
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
+	g_return_if_fail (painter != NULL);
+	g_return_if_fail (HTML_IS_PAINTER (painter));
+	g_return_if_fail (text != NULL);
+	g_return_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT);
 
-	return (* HP_CLASS (painter)->calc_ascent) (painter, font_style, face);
-}
-
-guint
-html_painter_calc_descent (HTMLPainter *painter,
-			   GtkHTMLFontStyle font_style,
-			   HTMLFontFace *face)
-{
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
-
-	return (* HP_CLASS (painter)->calc_descent) (painter, font_style, face);
-}
-
-guint
-html_painter_calc_text_width (HTMLPainter *painter,
-			      const gchar *text,
-			      guint len, gint *line_offset,
-			      GtkHTMLFontStyle font_style,
-			      HTMLFontFace *face)
-{
-	guint width;
-	gchar *translated;
-	gint translated_len;
-
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (text != NULL, 0);
-	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
-
-	translated = alloca (strlen (text) + calc_text_bytes_delta (text, len, *line_offset, &translated_len,
-								    *line_offset != -1) + 1);
-	*line_offset = translate_text_special_chars (text, translated, len, *line_offset, *line_offset != -1);
-
-	width = (* HP_CLASS (painter)->calc_text_width) (painter, translated, translated_len, font_style, face);
-
-	return width;
+	(* HP_CLASS (painter)->calc_text_size) (painter, text, len, items, glyphs, font_style, face, width, asc, dsc);
 }
 
 static gint
@@ -415,23 +353,20 @@ correct_width (const gchar *text, guint bytes_len, gint *lo, HTMLFont *font)
 	return delta;
 }
 
-guint
-html_painter_calc_text_width_bytes (HTMLPainter *painter,
+void
+html_painter_calc_text_size_bytes (HTMLPainter *painter,
 				    const gchar *text,
-				    guint bytes_len, gint *line_offset,
-				    HTMLFont *font, GtkHTMLFontStyle style)
+				    guint bytes_len, GList *items, GList *glyphs, gint *line_offset,
+				    HTMLFont *font, GtkHTMLFontStyle style,
+				    gint *width, gint *asc, gint *dsc)
 {
-	guint width;
+	g_return_if_fail (painter != NULL);
+	g_return_if_fail (HTML_IS_PAINTER (painter));
+	g_return_if_fail (text != NULL);
+	g_return_if_fail (style != GTK_HTML_FONT_STYLE_DEFAULT);
 
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (text != NULL, 0);
-	g_return_val_if_fail (style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
-
-	width = (* HP_CLASS (painter)->calc_text_width_bytes) (painter, text, bytes_len, font, style);
-	width += correct_width (text, bytes_len, line_offset, font);
-
-	return width;
+	(* HP_CLASS (painter)->calc_text_size_bytes) (painter, text, bytes_len, items, glyphs, font, style, width, asc, dsc);
+	*width += correct_width (text, bytes_len, line_offset, font);
 }
 
 /* The actual paint operations.  */
@@ -469,115 +404,37 @@ html_painter_draw_rect (HTMLPainter *painter,
 	(* HP_CLASS (painter)->draw_rect) (painter, x, y, width, height);
 }
 
-static gint
-calc_text_bytes_delta (const gchar *text, gint len, gint line_offset, gint *translated_len, gboolean tabs)
+void
+html_replace_tabs (const gchar *text, gchar *translated, guint bytes)
 {
-	gunichar uc;
-	const gchar *s;
-	gint delta, skip, current_len;
+	const gchar *t, *tab;
+	gchar *tt;
 
-	current_len = 0;
-	delta = 0;
-	*translated_len = 0;
-	s = text;
-	while (s && (uc = g_utf8_get_char (s)) && current_len < len) {
-		switch (uc) {
-		case ENTITY_NBSP:
-			delta --;
-			(*translated_len) ++;
-			line_offset ++;
-			break;
-		case '\t':
-			if (tabs) {
-				skip = 8 - (line_offset % 8);
-				delta += skip - 1;
-				line_offset += skip;
-				(*translated_len) += skip;
-			} else {
-				(*translated_len) ++;
-				line_offset ++;
-			}
-			break;
-		default:
-			(*translated_len) ++;
-			line_offset ++;
-		}
-		current_len ++;
-		s = g_utf8_next_char (s);
-	}
+	t = text;
+	tt = translated;
 
-	return delta;
-}
-
-static inline void
-put_last (const gchar *s, const gchar **ls, gchar **translated)
-{
-	if (*ls)
-		for (; *ls < s; (*ls) ++, (*translated) ++)
-			**translated = **ls;
-}
-
-static gint
-translate_text_special_chars (const gchar *text, gchar *translated, gint len, gint line_offset, gboolean tabs)
-{
-	gunichar uc;
-	const gchar *s, *ls;
-	gint skip, current_len;
-
-	current_len = 0;
-	s = text;
-	ls = NULL;
-	while (s && (uc = g_utf8_get_char (s)) && current_len < len) {
-		put_last (s, &ls, &translated);
-		switch (uc) {
-		case ENTITY_NBSP:
-			*translated = ' ';
-			translated ++;
-			line_offset ++;
-			ls = NULL;
-			break;
-		case '\t':
-			if (tabs) {
-				skip = 8 - (line_offset % 8);
-				line_offset += skip;
-				for (; skip; skip --, translated ++)
-					*translated = ' ';
-			} else {
-				*translated = ' ';
-				translated ++;
-				line_offset ++;
-			}
-			ls = NULL;
-			break;
-		default:
-			ls = s;
-			line_offset ++;
-		}
-		current_len ++;
-		s = g_utf8_next_char (s);
-	}
-	put_last (s, &ls, &translated);
-	*translated = 0;
-
-	return line_offset;
+	do {
+		tab = memchr (t, (unsigned char) '\t', bytes - (t - text));
+		if (tab) {
+			strncpy (tt, t, tab - t);
+			tt += tab - t;
+			*tt = ' ';
+			tt ++;
+			t = tab + 1;
+		} else
+			strncpy (tt, t, bytes - (t - text));
+	} while (tab);
 }
 
 gint
 html_painter_draw_text (HTMLPainter *painter,
 			gint x, gint y,
-			const gchar *text, gint len, gint line_offset)
+			const gchar *text, gint len, GList *items, GList *glyphs, gint line_offset)
 {
-	gchar *translated;
-	gint translated_len;
-
 	g_return_val_if_fail (painter != NULL, line_offset);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), line_offset);
 
-	translated = alloca (strlen (text) + calc_text_bytes_delta (text, len, line_offset, &translated_len,
-								    line_offset != -1) + 1);
-	line_offset = translate_text_special_chars (text, translated, len, line_offset, line_offset != -1);
-
-	(* HP_CLASS (painter)->draw_text) (painter, x, y, translated, translated_len);
+	(* HP_CLASS (painter)->draw_text) (painter, x, y, text, len, items, glyphs);
 
 	return line_offset;
 }
@@ -713,33 +570,45 @@ gint
 html_painter_draw_spell_error (HTMLPainter *painter,
 			       gint x, gint y,
 			       const gchar *text,
-			       gint len)
+			       gint len, GList *items, GList *glyphs)
 {
-	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len);
+	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len, items, glyphs);
 }
 
 HTMLFont *
-html_painter_alloc_font (HTMLPainterClass *pc, gchar *face_name, gdouble size, gboolean points, GtkHTMLFontStyle style)
+html_painter_alloc_font (HTMLPainter *painter, gchar *face_name, gdouble size, gboolean points, GtkHTMLFontStyle style)
 {
-	return (* pc->alloc_font) (face_name, size, points, style);
+	return (* HP_CLASS (painter)->alloc_font) (painter, face_name, size, points, style);
 }
 
 void
-html_painter_ref_font (HTMLPainterClass *pc, HTMLFont *font)
+html_painter_ref_font (HTMLPainter *painter, HTMLFont *font)
 {
-	(* pc->ref_font) (font);
+	(* HP_CLASS (painter)->ref_font) (painter, font);
 }
 
 void
-html_painter_unref_font (HTMLPainterClass *pc, HTMLFont *font)
+html_painter_unref_font (HTMLPainter *painter, HTMLFont *font)
 {
-	(* pc->unref_font) (font);
+	(* HP_CLASS (painter)->unref_font) (painter, font);
 }
 
 guint
 html_painter_get_space_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
 {
-	return get_html_font (painter, face, style)->space_width;
+	return html_font_manager_get_font (&painter->font_manager, face, style)->space_width;
+}
+
+guint
+html_painter_get_block_indent_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style)->indent_width;
+}
+
+guint
+html_painter_get_block_cite_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style)->cite_width;
 }
 
 guint
@@ -759,24 +628,3 @@ html_painter_set_focus (HTMLPainter *p, gboolean focus)
 {
 	p->focus = focus;
 }
-
-HTMLPainterClass *
-html_painter_class_from_id (HTMLFontManagerId id)
-{
-	switch (id) {
-	case HTML_FONT_MANAGER_ID_GDK:
-		return gtk_type_class (html_gdk_painter_get_type ());
-	case HTML_FONT_MANAGER_ID_PLAIN:
-		return gtk_type_class (html_plain_painter_get_type ());
-	case HTML_FONT_MANAGER_ID_PRINTER:
-		return gtk_type_class (html_printer_get_type ());
-	default:
-		return NULL;
-	}
-}
-
-HTMLFontManagerId
-html_painter_get_font_manager_id (HTMLPainter *painter)
-{
-	return 	(* HP_CLASS (painter)->get_font_manager_id) ();
-}
