@@ -23,7 +23,6 @@
 #ifndef _HTMLTEXT_H_
 #define _HTMLTEXT_H_
 
-#include <gal/unicode/gunicode.h>
 #include "htmlobject.h"
 
 #define HTML_TEXT(x) ((HTMLText *)(x))
@@ -51,6 +50,7 @@ struct _HTMLText {
 	guint select_length;
 
 	GList *spell_errors;
+	GList *items;
 };
 
 struct _HTMLTextClass {
@@ -115,6 +115,8 @@ gunichar          html_text_get_char                     (HTMLText         *text
 							  guint             offset);
 gchar            *html_text_get_text                     (HTMLText         *text,
 							  guint             offset);
+GList *           html_text_get_items                    (HTMLText         *text,
+							  HTMLPainter      *painter);
 void              html_text_spell_errors_clear           (HTMLText         *text);
 void              html_text_spell_errors_clear_interval  (HTMLText         *text,
 							  HTMLInterval     *i);
@@ -133,7 +135,7 @@ gboolean          html_text_convert_nbsp                 (HTMLText         *text
 gint              html_text_get_line_offset              (HTMLText         *text,
 							  HTMLPainter      *painter);
 gint              html_text_text_line_length             (const gchar      *text,
-							  gint              line_offset,
+							  gint             *line_offset,
 							  guint             len);
 /*
  * protected
